/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Load and display images and access pixel values. The pointillize function
 * is based on the Pointillism example sketch by Daniel Shiffman 
 * (https://processing.org/examples/pointillism.html)
 *****************************************************************************/

PImage sourceImg;  // Processing image object

int mode = 0;      // mode index
int numModes = 4;

int pointMinSize = 3;   // minimum size of pointillize circles
int pointMaxSize = 13;  // maximum size of pointillize circles

int strokeHalfSize = 9; // half size of of strokillize lines


void setup() {
  size(512, 512);
  
  sourceImg = loadImage("lena.png"); // load image and create PImage object.
                                     // "lena.png" must be in the sketch's "data" directory
}


void draw() {
  
  if (mode == 0) {
    // draw image with top-left corner at position x=0, y=0 and width and height equal to 
    // the application window size
    image(sourceImg, 0, 0, width, height);
  }
  else if (mode == 1) {
    // draw a pyramid of images with decreasing drawing size under the mouse pointer 
    imagePyramid(sourceImg, mouseX, mouseY);
  }
  else if (mode == 2) {
    // instead of drawing the image, draw points with colors according to pixel colors
    pointillize(sourceImg); 
  }
  else if (mode == 3) {
    // instead of drawing the image, draw lines with colors according to pixel colors
    strokillize(sourceImg); 
  }
  
}


void imagePyramid(PImage img, int x, int y) {
  // image drawing size. start with original size
  int w = img.width;
  int h = img.height;
  
  // as long as drawing size is larger than 1 pixel, draw image and then halfen the drawing size
  while (w > 1 && h > 1) {
    image(sourceImg, x*(width-w)/width, y*(height-h)/height, w, h);
    w /= 2;
    h /= 2;
  }
}


void pointillize(PImage img) {
  float pointsize;
  int x, y, i;
  color clr;
  
  ellipseMode(CENTER);
  
  // draw 20 circles per frame
  for (i = 0; i < 20; i++) {
    // get random point size
    pointsize = random(pointMinSize, pointMaxSize); 
    
    // get random image coordinate
    x = int(random(img.width));   
    y = int(random(img.height));
    
    // get pixel color at the yielded coordinate
    clr = img.get(x, y);
    
    // map image coordinate to app window coordinate (no change if image and window are of same size)
    x = (int)map(x, 0, img.width, 0, width);
    y = (int)map(y, 0, img.height, 0, height);
    
    // draw circle with color equal to pixel color and alpha=128
    noStroke();
    fill(clr, 128);
    ellipse(x, y, pointsize, pointsize);
  }
}


void strokillize(PImage img) {
  float angle;
  int x, y, i;
  float x0, y0;
  color clr;
  
  ellipseMode(CENTER);
  
  // paint a almost transparent black rectangle over the last frame
  // to slowly fade out lines from previous frames
  noStroke();
  fill(0, 0, 0, 2);
  rect(0, 0, width, height);
  
  // draw 200 lines per frame
  for (i = 0; i < 200; i++) {
    // compute random line angle
    angle = random(PI);
    
    // compute random image coordinate
    x = int(random(img.width));
    y = int(random(img.height));
    
    // get pixel color at yielded coordinate
    clr = img.get(x, y);
    
    // map image coordinate to app window coordinate (no change if image and window are of same size)
    x = (int)map(x, 0, img.width, 0, width);
    y = (int)map(y, 0, img.height, 0, height);
    
    // compute coordinate that is 'strokeHalfSize' pixels away from the yielded coordinate at the computed random angle
    x0 = x + strokeHalfSize*sin(angle);
    y0 = y + strokeHalfSize*cos(angle);
    
    // draw line from (x0,y0) to its reflection point reflected at (x, y)
    strokeWeight(1);
    stroke(clr, 128);
    line(2*x - x0, 2*y - y0, x0, y0);
  }
}







void mouseClicked() {

  if (mouseButton == LEFT) {
    background(0);
    mode = (mode + 1) % numModes;
  }
}
